<?php
namespace FiloBlu\Flow\Model\Channel\In;

abstract class AbstractModel {

    protected $_name;
    protected $_data;
    protected $_connector;
    protected $_parser;
    public $map;
    protected $_config;
    protected $_configModel;
    protected $_file;
    protected $_insertQueue = array();
    protected $_tablename;
    /**
    * @var Monolog\Logger
    */
    protected $_logger;


    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Channel\Config $channelConfig
    ) {
        $this->_logger = $loggerProvider->getLogger();
        $this->_configModel = $channelConfig;
    }


    public function init($channelname) {
        $this->loadConfig($channelname);
    }


    public function insertData($data) {

        // Genero il FLusso ITEMS
        $model =  \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model' . "\\" . $this->getConfig('flow'));

        if (!$model)
            return false;

        $model->setData('meta_file',$this->getFile()->getId());
        foreach($data as $field => $value){
            $fieldMap = $this->map->get($field);
            if(!isset($fieldMap["skip"])) {
                $to_field = $fieldMap["to"];
                $model->setData($to_field,$value);
            }
        }
        $model->save();

    }


    public function setName($name) {
        $this->_name = $name;
    }


    public function getName() {
        return $this->_name;
    }


    public function getConfig($key) {
        if(isset($this->_config[$key])) {
            return $this->_config[$key];
        }
        return null;
    }


    public function getData($key) {
        if(isset($this->_data[$key])) {
            return $this->_data[$key];
        }
        return null;
    }


    public function load($config) {
        $this->_data = (Array) $config;
    }


    public function storeData() {

    }


    public function setConnector($connector) {

        $connector_type = $connector['type'];

        $connector_class = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Connector'. "\\" . ucfirst($connector_type));

        if (trim($connector_type) and ($connector_class instanceof \FiloBlu\Flow\Model\Connector\AbstractConnector)) {
            $this->_connector = $connector_class;
            $this->_connector->setConfig($connector);
        }
        else
            throw new \Exception('Error in connector config');

    }


    public function getConnector() {
        return $this->_connector;
    }


    public function setParser($parser) {

        $parser_type = $parser['type'];

        $parser_class = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Parser'. "\\" . ucfirst($parser_type));

        if (trim($parser_type) and ($parser_class instanceof \FiloBlu\Flow\Model\Parser\AbstractParser)) {
            $this->_parser = $parser_class;
            $this->_parser->setConfig($parser);
        }

        else
            throw new \Exception('Error in parser config');

    }

    public function getParser() {
        return $this->_parser;
    }


    public function processBeginningOfFlow() {

    }


    public function processEndOfFlow() {

    }


    public function rlist(){

        $this->_logger->info(__METHOD__);

        $depends_resources = 0;

        if (!$this->_connector)
            throw new \Exception('no connector joined to channel');

        if($this->getData("depends_folder")){
            $depends_resources = $this->_connector->listDir($this->getData("depends_folder"));
            if(count($depends_resources) > 0 ){
                return false;
            }
        }

        if($this->getData("depends_file")){
            $depends_resources = $this->_connector->getResources();;
            if(count($depends_resources) > 0 ){
                return false;
            }
        }

        $resources = $this->_connector->getResources();

        if (!count($resources))
            return false;

        return $resources;

    }


    public function receiveFile($file){

        $this->_logger->info(__METHOD__);

        if (!$this->_connector)
            throw new \Exception('no connector joined to channel');

        $received = $this->_connector->receiveResource($file->getName());

        return $received;

    }

    public function parse($file) {

        if (!$this->_connector)
            throw new \Exception('no connector joined to channel');

        if (!$this->_parser)
            throw new \Exception('no parser joined to channel');
        $this->setFile($file);
        $localdir = $this->_connector->getLocalDir();

        $this->_parser->loadMap($file->getChannel());
        $this->setMap($this->_parser->map);
        $this->_parser->setChannel($this);

        //Il Parser chiama il channel con il metodo insertData
        try {
            $this->_parser->process($localdir . "/" . $file->getName());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
        return true;

    }


    public function setFile($file){
        $this->_file = $file;
        return $this;
    }


    public function getFile(){
        return $this->_file;
    }


    public function processendingofFlow() {

    }


    public function setMap($map){
        $this->map = $map;
        return $this;
    }


    protected function _getWebsitesIds(){
        if(!isset($this->websites)) {
            $websitesIds = array();
            $websites = Mage::app()->getWebsites();
            foreach ($websites as $website) {
                $websitesIds[] = $website->getId();
            }
            $this->websites = implode(",",$websitesIds);
        }
        return $this->websites;
    }


    protected function _getStoreIdByCode($code){
        if(!isset($this->stores)){
            $stores= Mage::app()->getStores();
            $stores_by_code = array();
            foreach($stores as $store){
                $stores_by_code[$store->getCode()] = $store->getId();
            }
            $this->stores = $stores_by_code;
        }
        return $this->stores[$code];
    }


    public function commitData(){
        return true;
    }


    public function loadConfig($name){

        $this->_logger->info($name);

        // $name -> DB flow_channels.name
        // unique value (string)

        $this->_configModel->load($name);

        if($this->_configModel->initialized){
            $this->_config = $this->_configModel->get('config');
            $this->setConnector($this->_configModel->get('connector'));
            $this->setParser($this->_configModel->get('parser'));
            return true;
        }
        return false;
    }

}
